#region License
/* ****************************************************************************
 * Copyright (c) Llewellyn Pritchard. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. 
 * A copy of the license can be found in the License.html file at the root of this distribution. 
 * By using this source code in any fashion, you are agreeing to be bound by the terms of the 
 * Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 * ***************************************************************************/
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using Mast = Microsoft.Scripting.Ast.Ast;
using Arg = Microsoft.Scripting.Ast.Arg;
using Microsoft.Scripting;
using IronLisp.Compilers;
using System.Reflection;
using IronLisp.Runtime;
using Microsoft.Scripting.Ast;
using Microsoft.Scripting.Utils;
using System.Collections;
using Microsoft.Scripting.Hosting;
using IronLisp.Hosting;
using Microsoft.Scripting.Actions;
using System.Diagnostics;
using Microsoft.Scripting.Types;

using MSA = Microsoft.Scripting.Ast;


namespace IronLisp.Ast
{
  public class ImproperList : ExpressionList
  {
    public override MSA.Expression GetCons(CodeBlock cb)
    {
      MSA.Expression ex = base.GetCons(cb);

      return Mast.Call(null, typeof(Builtins).GetMethod("ToImproper"), ex);
    }

    public override Microsoft.Scripting.Ast.Expression GetAst(CodeBlock cb)
    {
      return base.GetAst(cb);
    }

    public new object Rest()
    {
      if (Count <= 1)
      {
        return null;
      }

      if (Count == 2)
      {
        return this[1];
      }
      ImproperList l = new ImproperList();
      for (int i = 1; i < this.Count; i++)
      {
        l.Add(this[i]);
      }
      return l;
    }

    public override string ToString()
    {
      List<string> l = new List<string>();
      foreach (Expression expr in this)
      {
        l.Add(expr.ToString());
      }

      l.Insert(l.Count - 1, ".");

      return "(" + string.Join(" ", l.ToArray()) + ")";
    }
  }

  public class ExpressionList : Expression, IEnumerable<Expression>
  {
    public ExpressionList()
    {

    }

    public ExpressionList(params Expression[] exprs)
    {
      exprlist.AddRange(exprs);
    }

    List<Expression> exprlist = new List<Expression>();

    public void Add(Expression expr)
    {
      exprlist.Add(expr);
    }

    public void AddRange(IEnumerable<Expression> exprs)
    {
      foreach (Expression exp in exprs)
      {
        Add(exp);
      }
    }

    public override string ToString()
    {
      List<string> l = new List<string>();
      foreach (Expression expr in exprlist)
      {
        l.Add(expr.ToString());
      }

      return "(" + string.Join(" ", l.ToArray()) + ")";
    }

    public Expression this[int index]
    {
      get { return exprlist[index]; }
      internal set { exprlist[index] = value; }
    }

    public int Count
    {
      get { return exprlist.Count; }
    }

    public IEnumerator<Expression> GetEnumerator()
    {
      foreach (Expression expr in exprlist)
      {
        yield return expr;
      }
    }

    System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
    {
      return GetEnumerator();
    }

    public override MSA.Expression GetCons(CodeBlock cb)
    {
      bool hassplice = false;
      foreach (Expression expr in exprlist)
      {
        if (expr is UnquoteSplicing)
        {
          hassplice = true;
          break;
        }
      }
      if (hassplice)
      {
        MSA.Expression[] aa = new MSA.Expression[exprlist.Count];

        for (int i = 0; i < exprlist.Count; i++)
        {
          if (exprlist[i] is UnquoteSplicing)
          {
            aa[i] = exprlist[i].GetCons(cb);
          }
          else
          {
            MSA.Expression ex = exprlist[i].GetCons(cb);
            aa[i] = Mast.Call(null, MakeList(ex), ex);
          }
        }

        MSA.Expression r = Mast.Call(null, typeof(Builtins).GetMethod(
          this is VectorDef ? "VectorAppend" : "Append"
          , new Type[] { typeof(object[]) }), aa);

        return r;


      }
      else
      {
        MSA.Expression[] aa = exprlist.ConvertAll<MSA.Expression>(
             delegate(Expression e) { return e.GetCons(cb); }).ToArray();

        if (this is VectorDef)
        {
          return Mast.NewArray(typeof(object[]), aa);
        }
        else
        {
          return Mast.Call(null, MakeList(aa), aa);
        }
      }
    }

    public virtual ExpressionList Rest()
    {
      if (Count == 0)
      {
        return null;
      }
      ExpressionList l = new ExpressionList();
      for (int i = 1; i < exprlist.Count; i++)
      {
        l.Add(exprlist[i]);
      }
      return l;
    }

    public override MSA.Expression GetAst(CodeBlock cb)
    {
      List<Expression> args = new List<Expression>(exprlist);

      if (args.Count > 0)
      {
        // hmmm, what we doing could possibly be used for 'syntax-case'

        args.RemoveAt(0);

        Symbol s = this[0] as Symbol;
        bool islocal = false;

        // deal with macros

        if (s != null)
        {
          object md;
          if (Compiler.Scope.TryLookupName(SymbolTable.StringToId(s.Name), out md))
          {
            Runtime.Macro ctcn = md as Runtime.Macro;

            if (ctcn != null)
            {
              Debug.WriteLine(this, "macro:input");
              Expression[] aa = args.ToArray();
              object result = ctcn.Call(Compiler, aa);

              Debug.WriteLine(result, "macro:output");

              if (result == null)
              {
                return Mast.Null();
              }

              Expression expr = IronLispLanguageContext.ParseExpressionString(result.ToString());
              return expr.GetAst(cb);
            }
          }
        }

        if (s != null && s.Name == "macroexpand" && args.Count > 0)
        {
          Symbol mname = args[0] as Symbol;

          if (mname != null)
          {


            object md;
            if (Compiler.Scope.TryLookupName(SymbolTable.StringToId(mname.Name), out md))
            {
              Runtime.Macro ctcn = md as Runtime.Macro;
              args.RemoveAt(0);
              object result = ctcn.Call(Compiler, args.ToArray());

              Expression expr = IronLispLanguageContext.ParseExpressionString("'" + result.ToString());

              return expr.GetAst(cb);
            }
          }
          else // old style
          {
            Quote q = args[0] as Quote;
            List<Expression> l = new List<Expression>(q.Expression as ExpressionList);
            mname = l[0] as Symbol;

            if (mname != null)
            {
              object md;
              if (Compiler.Scope.TryLookupName(SymbolTable.StringToId(mname.Name), out md))
              {
                Runtime.Macro ctcn = md as Runtime.Macro;
                l.RemoveAt(0);
                object result = ctcn.Call(Compiler, l.ToArray());

                Expression expr = IronLispLanguageContext.ParseExpressionString("'" + result.ToString());

                return expr.GetAst(cb);
              }
            }
          }
        }

        // lets see if its variable maybe
        // also necessary?
        if (s != null)
        {
          Variable local = FindVar(cb, (SymbolId)s.Name);

          islocal = local != null;

          Type lt = GetVariableType(s.Name, cb);

          if (local != null && typeof(Delegate).IsAssignableFrom(lt) && lt != typeof(Delegate))
          {
            MSA.Expression[] aa = args.ConvertAll<MSA.Expression>(
                delegate(Expression e) { return e.GetAst(cb); }).ToArray();

            aa = ArrayUtils.Insert<MSA.Expression>(Mast.CodeContext(), aa);

            return
              Mast.Call(Mast.Read(local), lt.GetMethod("Invoke"), aa);
          }
        }

        if (s != null && builtinmap.ContainsKey(s.Name) && !islocal)
        {
          MSA.Expression[] aa = args.ConvertAll<MSA.Expression>(
            delegate(Expression e) { return e.GetAst(cb); }).ToArray();

          Type[] argtypes = Array.ConvertAll<MSA.Expression, Type>(aa, delegate(MSA.Expression e) { return e.Type; });

          BuiltinFunction bf = builtinmap[s.Name];
          MethodBinder mb = MethodBinder.MakeBinder(BINDER, s.Name, bf.Targets, BinderType.Normal);

          MethodCandidate mc = mb.MakeBindingTarget(CallType.None, argtypes);

          if (mc != null)
          {
            if (mc.Target.NeedsContext)
            {
              aa = ArrayUtils.Insert<MSA.Expression>(Mast.CodeContext(), aa);
            }

            MethodBase target = mc.Target.Method;

            OptimizeParameters(aa, target);

            return
              Mast.Call(null, mc.Target.Method as MethodInfo, aa);
          }
        }
        // check static/instance method/property
        else if (args.Count > 0 && s != null && !islocal)
        {
          BindingFlags si = 0;

          Symbol t = args[0] as Symbol;
          Type target = t == null ? null : TypeResolver.GetType(t.Name);
          MSA.Expression instance = null;
          if (t == null || target == null)
          {
            si = BindingFlags.Instance;
            instance = args[0].GetAst(cb);
            target = instance.Type;
          }
          else
          {
            si = BindingFlags.Static;
          }
          if (target != null)
          {
            List<MethodBase> cans = new List<MethodBase>();
            foreach (MemberInfo mi in target.GetMember(s.Name, si | BindingFlags.Public | BindingFlags.FlattenHierarchy | BindingFlags.IgnoreCase))
            {
              PropertyInfo pi = mi as PropertyInfo;
              if (pi != null)
              {
                // todo property set
                return Mast.ReadProperty(instance, target, pi.Name);
              }

              FieldInfo fi = mi as FieldInfo;
              if (fi != null)
              {
                if (args.Count > 1)
                {
                  return Mast.AssignField(instance, target, fi.Name, args[1].GetAst(cb));
                }
                else
                {
                  return Mast.ReadField(instance, target, fi.Name);
                }
              }

              MethodInfo methi = mi as MethodInfo;
              if (methi != null)
              {
                {
                  cans.Add(methi);
                }
              }
            }

            if (cans.Count > 0)
            {
              MethodBinder mb =
                MethodBinder.MakeBinder(new IronLisp.Actions.IronLispActionBinder(null), s.Name, cans.ToArray(), BinderType.Normal);

              CallType ct = CallType.ImplicitInstance;

              if (si == BindingFlags.Static)
              {
                ct = CallType.None;
                args.RemoveAt(0);
              }

              MSA.Expression[] aa = args.ConvertAll<MSA.Expression>(
                delegate(Expression e)
                {
                  return e.GetAst(cb);
                }).ToArray();

              Type[] at = Array.ConvertAll<MSA.Expression, Type>(aa,
                delegate(MSA.Expression e) { return e.Type; });

              MethodCandidate mc = mb.MakeBindingTarget(ct, at);

              if (si == BindingFlags.Instance)
              {
                aa = ArrayUtils.RemoveFirst(aa);
              }

              if (mc != null)
              {
                OptimizeParameters(aa, mc.Target.Method);
                return
                  Mast.Call(instance, mc.Target.Method as MethodInfo, aa);
              }
            }
          }
        }




#if !I_FIGURE_THIS_OUT
        // maybe a function pointer?
        if (this[0] is Quote && args.Count == 1)
        {
          Quote f = this[0] as Quote;
          if (f.Expression is Symbol)
          {
            Symbol so = f.Expression as Symbol;

            MSA.Expression ex = args[0].GetAst(cb) as MSA.Expression;

            Type target = ex.Type;

            if (args[0] is Symbol)
            {
              target = TypeResolver.GetType(((Symbol)args[0]).Name);
            }

            MethodInfo mi = (target ?? ex.Type).GetMethod(so.Name, BindingFlags.Public | BindingFlags.Static | 
              BindingFlags.Instance | BindingFlags.IgnoreCase | BindingFlags.FlattenHierarchy);
            Type ct = ReflectionUtils.GetDelegateType(ReflectionUtils.GetParameterTypes(mi.GetParameters()), mi.ReturnType);

            ConstructorInfo ci = ct.GetConstructor(new Type[] { typeof(object), typeof(IntPtr) });

            return Mast.New(ci, ex, Mast.Constant(mi));
          }
        }
#endif

        MSA.Expression[] exargs = exprlist.ConvertAll<MSA.Expression>(
                delegate(Expression e)
                {
                  return e.GetAst(cb);
                }).ToArray();

        return Mast.Action.Call(typeof(object), exargs);

      }
      //Debug.Fail("TODO");
      return Mast.Null();// Void(Mast.Empty());
    }

    static void OptimizeParameters(MSA.Expression[] aa, MethodBase target)
    {
      ParameterInfo[] pis = target.GetParameters();

      for (int i = 0; i < pis.Length && i < aa.Length; i++)
      {

        if (aa[i].Type != pis[i].ParameterType && aa[i] is UnaryExpression)
        {
          UnaryExpression ue = (UnaryExpression)aa[i];
          if (ue.Operator == UnaryOperators.Convert && pis[i].ParameterType.IsAssignableFrom(ue.Operand.Type))
          {
            aa[i] = ue.Operand;
          }
        }
      }
    }

  }

  public interface IExpression
  {
    MSA.Expression GetAst(CodeBlock cb);
    MSA.Expression GetCons(CodeBlock cb);
    SourceSpan Span { get;}
  }


  public abstract class Expression : IExpression
  {
    public abstract MSA.Expression GetAst(CodeBlock cb);
    gppg.LexLocation location;

    protected internal readonly static Dictionary<string, BuiltinFunction> builtinmap =
      new Dictionary<string, BuiltinFunction>(StringComparer.InvariantCultureIgnoreCase);

    readonly static Dictionary<CodeBlock, Scope> scopemap = new Dictionary<CodeBlock, Scope>();

    static CodeBlock evalblock;

    static Expression()
    {
      IronLispScriptEngine se = ScriptDomainManager.CurrentManager.GetLanguageProvider(
        typeof(Hosting.IronLispLanguageProvider)).GetEngine() as IronLispScriptEngine;
      ModuleContext mc = new ModuleContext(ScriptDomainManager.CurrentManager.CreateModule("macros"));
      mc.CompilerContext = new CompilerContext(SourceUnit.CreateSnippet(se, ""));


      CC = se.CreateContext(mc);

      // builtin methods

      Dictionary<string, List<MethodBase>> all = new Dictionary<string, List<MethodBase>>();

      foreach (MethodInfo mi in typeof(Builtins).GetMethods())
      {
        foreach (BuiltinAttribute ba in mi.GetCustomAttributes(typeof(BuiltinAttribute), false))
        {
          string mn = ba.Name ?? mi.Name.ToLower();
          List<MethodBase> meths;
          if (!all.TryGetValue(mn, out meths))
          {
            all[mn] = meths = new List<MethodBase>();
          }
          meths.Add(mi);
        }
      }

      foreach (string mn in all.Keys)
      {
        builtinmap[mn] = BuiltinFunction.MakeMethod(mn, all[mn].ToArray(), FunctionType.Function);
      }
    }

    readonly static CodeContext CC;

    protected internal static CodeContext Compiler
    {
      get { return CC; }
    }

    protected static MSA.Expression Read(string name, CodeBlock cb, Type type)
    {
      SymbolId sname = SymbolTable.StringToId(name.ToLower());

      Scope scope = GetScope(cb);

      object cvalue;
      if (!scope.TryLookupName(sname, out cvalue))
      {
        scope.SetName(sname, cvalue = (type ?? typeof(object)));
      }

      Type t = cvalue as Type;

      Variable v = FindVar(cb, sname);

      if (v == null)
      {
        return Mast.Read(sname);
      }

      if (t.IsAssignableFrom(v.Type))
      {
        return Mast.Read(v);
      }
      else
      {
        return Mast.DynamicConvert(Mast.Read(v), t);
      }
    }

    protected static Variable Create(string name, CodeBlock cb, Type type)
    {
      if (cb.Name == "__toploop__")
      {
        if (evalblock == null)
        {
          evalblock = cb;
        }
        else
        {
          cb = evalblock;
        }
      }
      SymbolId sname = SymbolTable.StringToId(name.ToLower());
      Scope scope = GetScope(cb);

      object cvalue;
      if (!scope.TryGetName(sname, out cvalue))
      {
        scope.SetName(sname, cvalue = (type ?? typeof(object)));
      }

      Type t = cvalue as Type;

      if (t != type)
      {
        object ot;
        if (scope.TryGetName(sname, out ot))
        {
          scope.SetName(sname, type);
        }
      }

      Variable v = MakeVar(cb, sname, typeof(object));
      return v;
    }


    protected static Variable CreateParameter(string name, CodeBlock cb, Type type)
    {
      SymbolId sname = SymbolTable.StringToId(name.ToLower());
      Scope scope = GetScope(cb);

      object cvalue;
      if (!scope.TryGetName(sname, out cvalue))
      {
        scope.SetName(sname, cvalue = (type ?? typeof(object)));
      }

      Type t = cvalue as Type;

      if (t != type)
      {
        object ot;
        if (scope.TryGetName(sname, out ot))
        {
          scope.SetName(sname, type);
        }
      }

      Variable v = Variable.Parameter(cb, sname, typeof(object));
      cb.Parameters.Add(v);
      return v;
    }

    protected static Variable GetParameter(string name, CodeBlock cb)
    {
      SymbolId sname = SymbolTable.StringToId(name.ToLower());

      foreach (Variable par in cb.Parameters)
      {
        if (par.Name.CaseInsensitiveEquals(sname))
        {
          return par;
        }
      }
      return null;
    }

    static Scope GetScope(CodeBlock key)
    {
      if (key.Name == "__toploop__")
      {
        if (evalblock == null)
        {
          evalblock = key;
        }
        else
        {
          key = evalblock;
        }
      }
      Scope cs = null;
      if (!scopemap.TryGetValue(key, out cs))
      {
        Scope parent = null;
        if (key.Parent != null)
        {
          parent = GetScope(key.Parent);
        }
        scopemap[key] = cs = new Scope(parent, null);
      }
      return cs;
    }

    protected static Type GetVariableType(string name, CodeBlock cb)
    {
      SymbolId sname = SymbolTable.StringToId(name.ToLower());

      Variable par = GetParameter(name, cb);
      if (par != null)
      {
        return typeof(object);
      }

      Scope scope = GetScope(cb);

      object cvalue;
      if (scope.TryGetName(sname, out cvalue))
      {
        return cvalue as Type;
      }

      if (cb.Parent != null)
      {
        return GetVariableType(name, cb.Parent);
      }

      return null;

    }

    static Variable MakeVar(CodeBlock cb, SymbolId name, Type type)
    {
      return cb.CreateVariable(name, cb.IsGlobal ? Variable.VariableKind.Local : Variable.VariableKind.Local, type ?? typeof(object));
    }

    static Variable FindOrMakeVar(CodeBlock cb, SymbolId name, Type type)
    {
      Variable v = FindVar(cb, name);
      return v ?? cb.CreateVariable(name, cb.IsGlobal ? Variable.VariableKind.Local : Variable.VariableKind.Local, type ?? typeof(object));
    }

    protected static Variable FindVar(CodeBlock cb, SymbolId name)
    {

      // variables take precidence
      foreach (Variable v in cb.Variables)
      {
        if (v.Name.CaseInsensitiveEquals(name))
        {
          return v;
        }
      }

      foreach (Variable v in cb.Parameters)
      {
        if (v.Name.CaseInsensitiveEquals(name))
        {
          return v;
        }
      }

      if (cb.Parent != null)
      {
        return FindVar(cb.Parent, name);
      }
      return null;
    }

    static MethodBinder listbinder;

    protected static MethodInfo MakeList(params MSA.Expression[] args)
    {
      Type[] types = Array.ConvertAll<MSA.Expression, Type>(args,
        delegate(MSA.Expression e) { return e.Type; });
      if (listbinder == null)
      {
        BuiltinFunction l = builtinmap["list"];
        listbinder = MethodBinder.MakeBinder(BINDER, l.Name, l.Targets, BinderType.Normal);
      }

      return listbinder.MakeBindingTarget(CallType.None, types).Target.Method as MethodInfo;
    }

    protected static readonly ActionBinder BINDER = new IronLisp.Actions.IronLispActionBinder(null);

    static MethodBase[] GetMethods(Type type, string name)
    {
      List<MethodBase> meths = new List<MethodBase>();
      foreach (MemberInfo memi in type.GetMember(name, BindingFlags.Public | BindingFlags.Static |
        BindingFlags.FlattenHierarchy | BindingFlags.IgnoreCase))
      {
        if (memi is MethodInfo)
        {
          meths.Add(memi as MethodBase);
        }
      }

      return meths.ToArray();
    }

    static MethodInfo GetMethod(Type type, string name)
    {
      return type.GetMethod(name, BindingFlags.Static | BindingFlags.Public);
    }

    internal gppg.LexLocation Location
    {
      get { return location; }
      set { location = value; }
    }

    public SourceSpan Span
    {
      get
      {
        return CreateSpan(location);
      }
    }

    [CLSCompliant(false)]
    protected static SourceSpan CreateSpan(gppg.LexLocation loc)
    {
      if (loc == null)
      {
        return SourceSpan.None;
      }
      return new SourceSpan(new SourceLocation(0, loc.sLin, loc.sCol + 1),
        new SourceLocation(1, loc.eLin, loc.eCol + 1));
    }

    protected static SourceSpan GetSpan(params IEnumerable<Expression>[] exprlists)
    {
      gppg.LexLocation loc = null;
      foreach (IEnumerable<Expression> exprlist in exprlists)
      {
        foreach (Expression e in exprlist)
        {
          if (loc == null)
          {
            loc = e.Location;
          }
          else
          {
            loc = loc.Merge(e.Location);
          }
        }
      }
      return CreateSpan(loc);
    }

    protected static string Join(IEnumerable<Expression> exprlist)
    {
      List<string> l = new List<string>();
      foreach (Expression e in exprlist)
      {
        l.Add(e.ToString());
      }

      return string.Join(" ", l.ToArray());
    }

    public virtual MSA.Expression GetCons(CodeBlock cb)
    {
      return GetAst(cb);
    }
  }

  public class Literal : Expression
  {
    object value;

    public object Value
    {
      get { return this.value; }
    }

    public Literal(object value)
    {
      this.value = value;
    }

    public override string ToString()
    {
      return Builtins.WriteFormat(value);
    }

    public override MSA.Expression GetAst(CodeBlock cb)
    {
      return Mast.Constant(value);
    }
  }

  public class Symbol : Expression
  {
    string name;

    public string Name
    {
      get { return name; }
    }

    public Symbol(string name)
    {
      this.name = name;
    }

    public override string ToString()
    {
      return string.Format("{0}", name);
    }

    public override MSA.Expression GetCons(CodeBlock cb)
    {
      return Mast.Constant(SymbolTable.StringToId(name));
    }

    public override MSA.Expression GetAst(CodeBlock cb)
    {
      return Read(name, cb, null);
    }
  }

  public class VectorDef : ExpressionList
  {
    public VectorDef(ExpressionList content)
    {
      AddRange(content);
    }

    public override MSA.Expression GetAst(CodeBlock cb)
    {
      List<MSA.Expression> inits = new List<MSA.Expression>();
      foreach (Expression ex in this)
      {
        inits.Add(ex.GetCons(cb));
      }
      return Mast.NewArray(typeof(object[]), inits);
    }

    public override string ToString()
    {
      List<string> l = new List<string>();
      foreach (Expression expr in this)
      {
        l.Add(expr.ToString());
      }

      return "#(" + string.Join(" ", l.ToArray()) + ")";
    }
  }

  public abstract class SpecialOperator : Expression
  {
    protected readonly Expression expr;
    protected static int NestingLevel = 0, UnquoteLevel = 0;

    internal Expression Expression
    {
      get { return expr; }
    }

    public SpecialOperator(Expression expr)
    {
      this.expr = expr;
    }
  }

  public class UnquoteSplicing : SpecialOperator
  {
    public UnquoteSplicing(Expression expr)
      : base(expr)
    {
    }

    public override string ToString()
    {
      return string.Format(",@{0}", expr);
    }

    public override MSA.Expression GetCons(CodeBlock cb)
    {
      try
      {
        UnquoteLevel++;
        if (UnquoteLevel == NestingLevel)
        {
          if (expr is ExpressionList)
          {
            return IronLispLanguageContext.ParseExpressionString(expr.ToString()).GetAst(cb);
          }
          return expr.GetAst(cb);
        }
        else
        {
          MSA.Expression[] args = 
          { 
            Mast.Constant(SymbolTable.StringToId("unquote-splicing")),
            expr.GetCons(cb)  
          };

          return Mast.Call(null, MakeList(args), args);
        }
      }
      finally
      {
        UnquoteLevel--;
      }
    }

    public override MSA.Expression GetAst(CodeBlock cb)
    {
      return expr.GetCons(cb);
    }
  }


  public class Unquote : SpecialOperator
  {
    public Unquote(Expression expr)
      : base(expr)
    {
    }

    public override string ToString()
    {
      return string.Format(",{0}", expr);
    }


    public override MSA.Expression GetCons(CodeBlock cb)
    {
      try
      {
        UnquoteLevel++;
        if (UnquoteLevel == NestingLevel)
        {
          if (expr is ExpressionList)
          {
            return IronLispLanguageContext.ParseExpressionString(expr.ToString()).GetAst(cb);
          }
          return expr.GetAst(cb);
        }
        else
        {
          MSA.Expression[] args = 
          { 
            Mast.Constant(SymbolTable.StringToId("unquote")),
            expr.GetCons(cb)  
          };

          return Mast.Call(null, MakeList(args), args);
        }
      }
      finally
      {
        UnquoteLevel--;
      }
    }

    public override MSA.Expression GetAst(CodeBlock cb)
    {
      try
      {
        UnquoteLevel++;
        return expr.GetAst(cb);
      }
      finally
      {
        UnquoteLevel--;
      }
    }
  }


  public class Quasiquote : SpecialOperator
  {
    public Quasiquote(Expression expr)
      : base(expr)
    {
    }

    public override string ToString()
    {
      return string.Format("`{0}", expr);
    }


    public override MSA.Expression GetCons(CodeBlock cb)
    {
      try
      {
        NestingLevel++;

        MSA.Expression[] args = 
        { 
          Mast.Constant(SymbolTable.StringToId("quasiquote")),
          expr.GetCons(cb)  
        };

        return Mast.Call(null, MakeList(args), args);
      }
      finally
      {
        NestingLevel--;
      }
    }

    public override MSA.Expression GetAst(CodeBlock cb)
    {
      try
      {
        NestingLevel++;
        return expr.GetCons(cb);
      }
      finally
      {
        NestingLevel--;
        if (NestingLevel < 0)
        {
          NestingLevel = 0;
        }
      }
    }
  }

  public class Quote : SpecialOperator
  {
    public Quote(Expression expr)
      : base(expr)
    {
    }

    public override string ToString()
    {
      return string.Format("'{0}", expr);
    }

    public override MSA.Expression GetCons(CodeBlock cb)
    {
      MSA.Expression[] args = 
      { 
        Mast.Constant(SymbolTable.StringToId("quote")),
        expr.GetCons(cb) 
      };

      return Mast.Call(null, MakeList(args), args);
    }

    public override MSA.Expression GetAst(CodeBlock cb)
    {
      if (expr is Literal)
      {
        return Mast.Constant(SymbolTable.StringToId(((Literal)expr).ToString()));
      }
      try
      {
        NestingLevel = int.MaxValue / 2;
        return expr.GetCons(cb);
      }
      finally
      {
        NestingLevel = 0;
      }
    }
  }
}
