/* ****************************************************************************
 * Copyright (c) Llewellyn Pritchard. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. 
 * A copy of the license can be found in the License.html file at the root of this distribution. 
 * By using this source code in any fashion, you are agreeing to be bound by the terms of the 
 * Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 * ***************************************************************************/

%namespace IronLisp.Compilers.Stage1
%using IronLisp.Ast
%{

public ExpressionList parsed;

%} 

%union
{
  public ExpressionList list;
  public Expression elem;
  public string text;
}

%token LBRACE RBRACE LBRACK RBRACK QUOTE QUASIQUOTE UNQUOTE UNQUOTESPLICING VECTORLBRACE DOT
%token <text> SYMBOL LITERAL STRING INTEGER REAL CHARACTER 

%type <list> exprlist list file
%type <elem> expr specexpr

%start file

%%

file 
    : exprlist                                    { parsed = $1; }
    ;
    
list
    : LBRACE exprlist RBRACE                      { $$ = $2; $$.Location = @1.Merge(@3); }
    | LBRACK exprlist RBRACK                      { $$ = $2; $$.Location = @1.Merge(@3); }
    | LBRACE exprlist DOT expr RBRACE             { $$ = new ImproperList(); $$.AddRange($2); $$.Add($4); } 
    | specexpr expr                               { $$ = new ExpressionList($1, $2); $$.Location = @1.Merge(@2); }
        
    ;

exprlist
    :                                             { $$ = new ExpressionList(); }
    | exprlist expr                               { if ($2 != null) { if ($1 == null) { $1 = new ExpressionList(); } $1.Add($2); $$ = $1; }}
    ;       
    
expr
    : list                                        { $$ = $1;}
  /*  | DOT                                         { $$ = new Symbol("."); $$.Location = @1;}*/
    | SYMBOL                                      { $$ = new Symbol($1); $$.Location = @1;}
    | STRING                                      { $$ = new Literal($1.Trim('"')); $$.Location = @1;}
    | INTEGER                                     { $$ = new Literal(Convert.ToInt32($1));$$.Location = @1;}
    | LITERAL                                     { $$ = new Literal($1 == "#t" ? (object)true : ($1 == "#f" ? (object)false : null));$$.Location = @1;}
    | REAL                                        { $$ = new Literal(Convert.ToDouble($1));$$.Location = @1;}
    | CHARACTER                                   { $$ = new Literal($1[0]);$$.Location = @1;}
    | VECTORLBRACE exprlist RBRACE                { $$ = new VectorDef($2); $$.Location = @1.Merge(@3); }
    ; 

specexpr
    : QUOTE                                       { $$ = new Symbol("quote"); $$.Location = @1;}
    | UNQUOTESPLICING                             { $$ = new Symbol("unquote-splicing"); $$.Location = @1;}
    | QUASIQUOTE                                  { $$ = new Symbol("quasiquote"); $$.Location = @1;}
    | UNQUOTE                                     { $$ = new Symbol("unquote"); $$.Location = @1;}
    ;    

%%









    