/* ****************************************************************************
 * Copyright (c) Llewellyn Pritchard. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. 
 * A copy of the license can be found in the License.html file at the root of this distribution. 
 * By using this source code in any fashion, you are agreeing to be bound by the terms of the 
 * Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 * ***************************************************************************/



%namespace IronLisp.Compilers.Stage2
%using IronLisp.Ast
%{

public ExpressionList parsed;

static readonly string[] DEFAULTUSINGS = { null };
public List<string> usings = new List<string>(DEFAULTUSINGS);

public List<string> references = new List<string>();


%} 

%union
{
  public ExpressionList list;
  public Expression elem;
  public List<CondClause> condclauses;
  public List<CaseClause> caseclauses;
  public List<DoClause> doclauses;
}

%token LBRACE RBRACE QUOTE QUASIQUOTE UNQUOTESPLICING UNQUOTE BEGIN LAMBDA SET NEW USING IF  REFERENCE DELAY ARROW MACRO
%token OR AND NOT THROW IS COND CALL TRY THE DEFINEMACRO LET ELSE CASE LETSTAR DEFINE LETREC DO
%token <elem> IDENTIFIER LITERAL STRING INTEGER REAL CHARACTER EXPR DOT

%type <doclauses> doclauses 
%type <caseclauses> caseclauses 
%type <condclauses> condclauses 
%type <list> exprlist list bracedpair bracedpairlist identifiers elsetail exprlist_opt dotest
%type <list> definitions body forms defineformals lambdaformals toplevel
%type <elem> expr coreexpr expr_opt constant derivedexpr clrexpr definition variabledefinition form


%start program

%%

program
    : toplevel                                              { parsed = $1; }
    ;
    
toplevel
    : forms
    ;
    
forms
    :                                                      { $$ = new ExpressionList(); }
    | forms form                                           { $1.Add($2); $$ = $1; }
    ;
    
form
    : definition
    | expr
    | pragma
    ;  
    
definition
    : variabledefinition
    | LBRACE DEFINEMACRO IDENTIFIER LBRACE LAMBDA lambdaformals 
        exprlist RBRACE RBRACE                      { $$ = new Define($3,new Macro($6,$7)); $$.Location = @2.Merge(@4);}          
    | LBRACE DEFINEMACRO defineformals
        exprlist RBRACE                             { $$ = new Define($3[0],new Macro($3.Rest(),$4)); $$.Location = @2.Merge(@4);}          
    ;
    
defineformals
    : LBRACE identifiers IDENTIFIER RBRACE        { $2.Add($3); $$ = $2; }
    | LBRACE identifiers IDENTIFIER 
        DOT IDENTIFIER RBRACE                     { $2.Add($3); $2.Add(new Symbol(".")); $2.Add($5); $$ = $2;  }    
    ;
    
variabledefinition
    : LBRACE DEFINE IDENTIFIER expr RBRACE        { $$ = new Define($3,$4); $$.Location = @2.Merge(@3);}  
    | LBRACE DEFINE defineformals body RBRACE    { $$ = new Define($3[0], new Lambda($3.Rest(), $4)); $$.Location = @2.Merge(@4);}  
    ;
    
body
    : definitions exprlist                       { $$ = Utils.TransformDefinitions($1,$2); }
    ;
    
definitions
    :                                           { $$ = new ExpressionList(); }
    | definitions definition                    { $1.Add($2); $$ = $1; }
    ;                 
    
list
    : LBRACE exprlist_opt RBRACE                  { $$ = $2; $$.Location = @2; }
    | LBRACE exprlist_opt DOT expr RBRACE         { $$ = new ImproperList(); $$.AddRange($2); $$.Add($4); $$.Location = @2.Merge(@4); }
    ;
    
exprlist_opt
    :                                             { $$ = new ExpressionList(); }
    | exprlist                                    
    ;

exprlist
    : expr                                        { $$ = new ExpressionList($1); }
    | exprlist expr                               { $1.Add($2); $$ = $1; }
    ; 
    
         
    
expr
    : coreexpr                                    { $$ = $1;}
    | derivedexpr                                 { $$ = $1;}
    | clrexpr                                     { $$ = $1;}    
    | list                                        { $$ = $1;}
    | constant                                    { $$ = $1;}
    ;
    
constant
    : IDENTIFIER                                  { $$ = $1;} 
    | LITERAL                                     { $$ = $1;} 
    | EXPR                                        { $$ = $1;} 
    ; 
    
derivedexpr
    : LBRACE BEGIN exprlist RBRACE                { $$ = new Begin($3); $$.Location = @2.Merge(@3);}
    | LBRACE OR exprlist_opt RBRACE              { $$ = new Or($3); $$.Location = @2.Merge(@4);}
    | LBRACE AND exprlist_opt RBRACE             { $$ = new And($3); $$.Location = @2.Merge(@4);}
    | LBRACE NOT expr RBRACE                      { $$ = new Not($3); $$.Location = @2.Merge(@3);}
    | LBRACE CASE expr caseclauses elsetail RBRACE { $$ = new Case($3,$4,$5); $$.Location = @2.Merge(@5);}
    | LBRACE COND condclauses elsetail RBRACE     { $$ = new Cond($3,$4); $$.Location = @2.Merge(@4);}
    | LBRACE LETSTAR LBRACE bracedpairlist RBRACE
       body RBRACE                            { $$ = new LetStar($4,$6); $$.Location = @2.Merge(@6);}

    | LBRACE LET LBRACE bracedpairlist RBRACE
       body RBRACE                            { $$ = new Let($4,$6); $$.Location = @2.Merge(@6);}
       
    | LBRACE LETREC LBRACE bracedpairlist RBRACE
       body RBRACE                            { $$ = new LetRec($4,$6); $$.Location = @2.Merge(@6);}

    | LBRACE LET IDENTIFIER LBRACE bracedpairlist 
        RBRACE body RBRACE                    { $$ = new NamedLet($3,$5,$7); $$.Location = @2.Merge(@7);}
        
    | LBRACE DO LBRACE doclauses RBRACE
        dotest exprlist_opt RBRACE                { $$ = new Do($4,$6,$7); } 
    | LBRACE DELAY expr RBRACE                  { $$ = new Delay($3); }
    ;
    
clrexpr
    : LBRACE CALL IDENTIFIER expr exprlist_opt RBRACE      { $$ = new Call($3,$4,$5); $$.Location = @2.Merge(@4);}
    | LBRACE THROW expr_opt RBRACE                { $$ = new Throw($3); $$.Location = @2.Merge(@3);}
    | LBRACE IS IDENTIFIER expr RBRACE            { $$ = new Is($3,$4); $$.Location = @2.Merge(@4);}
    | LBRACE NEW IDENTIFIER exprlist_opt RBRACE   { $$ = new New($3,$4); $$.Location = @2.Merge(@4);}
    | LBRACE TRY expr expr expr_opt RBRACE        { $$ = new Try($3,$4,$5); $$.Location = @2.Merge(@5);}
    | LBRACE THE IDENTIFIER expr RBRACE           { $$ = new The($3,$4); $$.Location = @2.Merge(@4);}  
    ;
    
   
pragma
    : LBRACE USING IDENTIFIER RBRACE              { usings.Add(($3 as Symbol).Name); }    
    | LBRACE REFERENCE IDENTIFIER RBRACE          { references.Add(($3 as Symbol).Name); }    
    | LBRACE REFERENCE LITERAL RBRACE             { references.Add((string)($3 as Literal).Value); }     
    ;       
    
lambdaformals
    : IDENTIFIER                                  { $$ = new ExpressionList( new Symbol("."), $1); }
    | LBRACE identifiers RBRACE                   { $$ = $2; }
    | LBRACE identifiers IDENTIFIER DOT 
        IDENTIFIER RBRACE                         { $2.Add($3); $2.Add(new Symbol(".")); $2.Add($5); $$ = $2; }
    ;
    
    
coreexpr
    : LBRACE LAMBDA lambdaformals body RBRACE     { $$ = new Lambda($3,$4); $$.Location = @2.Merge(@4);}
    | LBRACE SET IDENTIFIER expr RBRACE           { $$ = new Set($3,$4); $$.Location = @2.Merge(@3);}
    | LBRACE IF expr expr expr_opt RBRACE         { $$ = new If($3,$4,$5); $$.Location = @2.Merge(@5);}
    | LBRACE MACRO lambdaformals body RBRACE      { $$ = new Macro($3,$4); $$.Location = @2.Merge(@4);}
    ;

dotest    
    : LBRACE expr exprlist_opt RBRACE             { $$ = new ExpressionList($2,$3); }
    ;
    
doclauses
    :                                             { $$ = new List<DoClause>(); }
    | doclauses LBRACE IDENTIFIER expr            
        expr_opt RBRACE                           { $1.Add( new DoClause($3,$4,$5)); $$ = $1; }
    ;    
    
     
enablereadcontent
    : { ((Scanner)scanner).readascontent = true; }
    ;         
     
disablereadcontent
    : { ((Scanner)scanner).readascontent = false; }
    ;         
    
elsetail
    : 
    | LBRACE ELSE exprlist RBRACE                 { $$ = $3; }
    ;
    
condclauses
    :                                             { $$ = new List<CondClause>(); }
    | condclauses LBRACE expr exprlist RBRACE     { $1.Add(new CondClause($3,$4)); $$ = $1; }
    | condclauses LBRACE expr ARROW expr RBRACE   { $1.Add(new CondClause($3,$5)); $$ = $1; }
    | condclauses LBRACE expr RBRACE              { $1.Add(new CondClause($3,null)); $$ = $1; }
    ; 
    
caseclauses
    :                                             { $$ = new List<CaseClause>(); }
    | caseclauses LBRACE enablereadcontent
        list disablereadcontent exprlist RBRACE   { $1.Add( new CaseClause($4,$6));$$ = $1; }
    ;     
    
   
identifiers
    :                                             { $$ = new ExpressionList(); }
    | identifiers expr                      { $1.Add($2); $$ = $1; }
    ;
    
 
bracedpair
    : LBRACE IDENTIFIER expr RBRACE               { $$ = new ExpressionList(); $$.Add($2); $$.Add($3); }
    ;
    
bracedpairlist
    :                                             { $$ = new ExpressionList(); }
    | bracedpairlist bracedpair                   { $1.Add($2); $$ = $1; }
    ;
    
expr_opt
    :
    | expr
    ;  


%%









    