#region License
/* ****************************************************************************
 * Copyright (c) Llewellyn Pritchard. 
 *
 * This source code is subject to terms and conditions of the Microsoft Permissive License. 
 * A copy of the license can be found in the License.html file at the root of this distribution. 
 * By using this source code in any fashion, you are agreeing to be bound by the terms of the 
 * Microsoft Permissive License.
 *
 * You must not remove this notice, or any other, from this software.
 * ***************************************************************************/
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using System.Diagnostics;
using Microsoft.Scripting;
using System.IO;

namespace IronLisp.Compilers
{
  static class TypeResolver
  {
    static List<string> usings;
    static readonly Dictionary<string, Assembly> references = new Dictionary<string, Assembly>();
    static readonly Dictionary<string, Type> typecache = new Dictionary<string, Type>();

    public static void AddReference(Assembly ass)
    {
      references[ass.GetName().Name] = ass;
    }

    public static void AddType(Type type)
    {
      typecache[type.Name.ToLower()] = type;
    }

    public static void Initialize(IEnumerable<string> uses, IEnumerable<string> refs)
    {
      typecache.Clear();
      references.Clear();
      references.Add("mscorlib", typeof(object).Assembly);
      //references.Add("System", typeof(Stopwatch).Assembly);
      references.Add("IronLisp", typeof(TypeResolver).Assembly);
      references.Add("Microsoft.Scripting", typeof(SymbolId).Assembly);
      usings = new List<string>();

      Add(uses, refs);
    }

    public static Type GetType(string typename)
    {
      typename = typename.ToLower();
      Type type;
      if (typecache.TryGetValue(typename, out type))
      {
        return type;
      }
      foreach (Assembly ass in references.Values)
      {
        foreach (Type t in ass.GetExportedTypes())
        {
          string ctname = t.Name.ToLower();
          if (ctname == typename)
          {
            foreach (string use in usings)
            {
              if (use == t.Namespace || (use != null && t.Namespace != null && use.ToLower() == t.Namespace.ToLower()))
              {
                return t;
              }
            }
          }
        }
      }
      return null;
    }

    public static void Add(IEnumerable<string> use, IEnumerable<string> refs)
    {
      usings.AddRange(use);

      foreach (string refer in refs)
      {
        if (refer.StartsWith("<") && refer.EndsWith(">"))
        {
          string rname = refer.Substring(1, refer.Length - 2);
          AddReference(Assembly.LoadFile(Path.GetFullPath(rname)));
        }
        else
        {
#pragma warning disable 618
          AddReference(Assembly.LoadWithPartialName(refer));
#pragma warning restore 618
        }
      }
    }
  }
}
