(using system.collections)

(= defmacro (macro (name args &rest body) 
  `(= ,name (macro ,args ,@body))))  

(defmacro defun (name args &rest body) 
  `(= ,name (fn ,args ,@body)))
  
; make me take scheme easier
(defmacro lambda (&rest body) 
  `(fn ,@body))


; this is rather ugly, but it allows for empty params, also setf style
(defmacro define (nargs &rest body)
  (if (is ienumerable `,nargs)
     (let ((name (car `,nargs))
           (args (cdr `,nargs)))
              (if (eq args null)
                `(defun ,name () ,@body)
                `(defun ,name ,args ,@body) ))   
     `(= ,nargs ,(car ,body)) ))
          
 
;;http://halogen.note.amherst.edu/~jdtang/scheme_in_48/tutorial/stdlib.html

(define (null? obj)         (eq obj null))
   
(define (id obj)             obj)

(define (flip func)         (lambda (arg1 arg2) (func arg2 arg1)))
(define (curry func arg1)   (lambda (arg) (apply func (cons arg1 arg))))
(define (compose f g)       (lambda (arg) (f (apply g arg))))

(define (zero?)             (curry eql 0))
(define (positive?)         (curry < 0))
(define (negative?)         (curry > 0))
(define (odd? num)          (eql (% num 2) 1))
(define (even? num)         (eql (% num 2) 0))

(define (foldr func end lst)
  (if (null? lst)
      end
      (func (car lst) (foldr func end (cdr lst)))))


(define (foldl func accum lst)
  (if (null? lst)
      accum
      (foldl func (func accum (car lst)) (cdr lst))))

;(prl (macroexpand define fold foldl)) ; macroexpand is a macro in it self, hence the quoted list is not needed
;(prl (macroexpand '(define fold foldl)))
(define fold foldl)
(define reduce fold)

(define (unfold func init pred)
  (if (pred init)
      (cons init null)
      (cons init (unfold func (func init) pred))))


(define (sum &rest lst)     (fold + 0 lst))
(define (product &rest lst) (fold * 1 lst))
(define (mand &rest lst)    (fold & true lst))
(define (mor &rest lst)     (fold | false lst))

(prl (product 1 2 3 4)); :)))

(define (max first &rest num-list) (fold (lambda (old n) (if (> old n) old n)) first num-list))
(define (min first &rest num-list) (fold (lambda (old n) (if (< old n) old n)) first num-list))

;(prl (max 1 4 8 5 72 3 5))
;(prl (min 4 8 5 72 3 5))

(define (mlength lst)        (fold (lambda (x y) (+ x 1)) 0 lst))

(prl (mlength "hello"))

(define (reverse lst)       (fold (flip cons) null lst))

(prl (reverse '(5 4 3 2 1)))

(define (mem-helper pred op) (lambda (acc next) (if (and (not acc) (pred (op next))) next acc)))
(define (memq obj lst)       (fold (mem-helper (curry eq obj) id) false lst))
(define (memv obj lst)       (fold (mem-helper (curry eql obj) id) false lst))
(define (member obj lst)     (fold (mem-helper (curry equal obj) id) false lst))
(define (assq obj alist)     (fold (mem-helper (curry eq obj) car) false alist))
(define (assv obj alist)     (fold (mem-helper (curry eql obj) car) false alist))
(define (assoc obj alist)    (fold (mem-helper (curry equal obj) car) false alist))

;(prl (memq 3 '(1 2 3 4 5)))
;(prl (memv 3 '(1 2 3 4 5)))
;(prl (member "3" '(1 2 3 4 5))) ; for equal we use a typeconvertor roundtrip test 

(define (map func lst)      (foldr (lambda (x y) (cons (func x) y)) null lst))

(define (filter pred lst)   (foldr (lambda (x y) (if (pred x) (cons x y) y)) null lst))

  
  
;(prl (filter even? '(1 2 3 4)))
;(prl (map (curry + 2) '(1 2 3 4)))


;;;;; backup ;;;;;;;;

#|
(define-syntax from
  (syntax-rules (in)
    ((from a c ... in d b) (let ((a b)) c ... d))))

(define-syntax cond
  (syntax-rules (else =>)
    ((cond (else result1 result2 ...))
     (begin result1 result2 ...))
    ((cond (test => result))
     (let ((temp test))
       (if temp (result temp))))
    ((cond (test => result) clause1 clause2 ...)
     (let ((temp test))
       (if temp
           (result temp)
           (cond clause1 clause2 ...))))
    ((cond (test)) test)
    ((cond (test) clause1 clause2 ...)
     (let ((temp test))
       (if temp
           temp
           (cond clause1 clause2 ...))))
    ((cond (test result1 result2 ...))
     (if test (begin result1 result2 ...)))
    ((cond (test result1 result2 ...)
           clause1 clause2 ...)
     (if test
         (begin result1 result2 ...)
         (cond clause1 clause2 ...)))))

|#

(define-macro (bug . clauses) `,(car ,clauses))
(let ()
    (define (g x) (+ x 2))
    (display (bug (g 1) (k 2)))
    (newline))



(define-macro (test a) 
  (let ((h (car a))
        (t (cdr a)))
    `'(hello ,a ,h ,@t)))
(prl    
(test (10 4 3))    
)

(define-macro (mylet a b . c)
  `((lambda (,a) ,@c) ,b))

(prl  
(macroexpand mylet s 1 (prl s)(prl 1))
)

(mylet e 5 (prl e))

(define myarr (new arraylist))



(add myarr 5)
(add myarr "hello")
(add myarr #\5)
    
(define true #t)
(define false #f) 


;;http://halogen.note.amherst.edu/~jdtang/scheme_in_48/tutorial/stdlib.html

(define (null? obj)         (eq? obj null))
   
(define (id obj)             obj)

(define (flip func)         (lambda (arg1 arg2) (func arg2 arg1)))
(define (curry func arg1)   (lambda (arg) (apply func (cons arg1 arg))))
(define (compose f g)       (lambda (arg) (f (apply g arg))))

;(define (zero?)             (curry eqv? 0))
;(define (positive?)         (curry < 0))
;(define (negative?)         (curry > 0))
;(define (odd? num)          (eqv? (% num 2) 1))
;(define (even? num)         (eqv? (% num 2) 0))

(define (foldr func end lst)
  (if (null? lst)
      end
      (func (car lst) (foldr func end (cdr lst)))))


(define (foldl func accum lst)
  (if (null? lst)
      accum
      (foldl func (func accum (car lst)) (cdr lst))))

;(prl (macroexpand define fold foldl)) ; macroexpand is a macro in it self, hence the quoted list is not needed
;(prl (macroexpand '(define fold foldl)))
(define fold foldl)
(define reduce fold)

(define (unfold func init pred)
  (if (pred init)
      (cons init null)
      (cons init (unfold func (func init) pred))))


(define (sum . lst)     (fold + 0 lst))
(define (product . lst) (fold * 1 lst))
;(define (mand &rest lst)    (fold & true lst))
;(define (mor &rest lst)     (fold | false lst))

;(prl (product 1 2 3 4)); :)))

(define (max first . num-list) (fold (lambda (old n) (if (> old n) old n)) first num-list))
(define (min first . num-list) (fold (lambda (old n) (if (< old n) old n)) first num-list))

(prl 
(max 1 4 8 5 72 3 5)
(min 4 8 5 72 3 5)
)

(define (mlength lst)        (fold (lambda (x y) (+ x 1)) 0 lst))

;(prl (mlength "hello"))

(define (reverse lst)       (fold (flip cons) null lst))

;(prl (reverse '(5 4 3 2 1)))

(define (mem-helper pred op) (lambda (acc next) (if (and (not acc) (pred (op next))) next acc)))
(define (memq obj lst)       (fold (mem-helper (curry eq? obj) id) #f lst))
(define (memv obj lst)       (fold (mem-helper (curry eqv? obj) id) #f lst))
(define (member obj lst)     (fold (mem-helper (curry equal? obj) id) #f lst))
(define (assq obj alist)     (fold (mem-helper (curry eq? obj) car) #f alist))
(define (assv obj alist)     (fold (mem-helper (curry eqv? obj) car) #f alist))
(define (assoc obj alist)    (fold (mem-helper (curry equal? obj) car) #f alist))

;(prl (memq 3 '(1 2 3 4 5)))
;(prl (memv 3 '(1 2 3 4 5)))
;(prl (member "3" '(1 2 3 4 5))) ; for equal we use a typeconvertor roundtrip test 

;#|
(define e '((a 1) (b 2) (c 3)))

(prl (assq 'a e))
(prl (assq 'b e))
(prl (assq 'd e))

(prl (assq (list 'a) '(((a)) ((b)) ((c)))))
(prl (assoc (list 'a) '(((a)) ((b)) ((c))))) ; spec says, must be list of pairs...

(prl (assq 5 '((2 3) (5 7) (11 13)))) ; hmmm this seems to pass because its using a cached constant, i wonder how i turn this off?
(prl (assv 5 '((2 3) (5 7) (11 13))))
(prl (assoc "5" '((2 3) (5 7) (11 13))))
;|#

(define (map func lst)      (foldr (lambda (x y) (cons (func x) y)) null lst))

(define (filter pred lst)   (foldr (lambda (x y) (if (pred x) (cons x y) y)) null lst))

  
  
;(prl (filter even? '(1 2 3 4)))
;(prl (map (curry + 2) '(1 2 3 4)))

;; r5rs functions

(define (boolean? obj)    (is boolean obj))
(define (symbol? obj)     (is symbolid obj))
(define (char? obj)       (is char obj))
(define (string? obj)     (is string obj))
(define (vector? obj)     (is ilist obj))
;(define (pair? obj)       (is boolean obj)) ; hmmm maybe i should support this
;(define (number? obj)     (mor (is int32 obj) (is double obj)))
;(define (port? obj)       (is boolean obj)) ; maybe make stream facade?
(define (procedure? obj)  (is fastcallable obj))
(define (list? obj)       (is ienumerable obj))


(define (list-tail x k)
  (if (zero? k)
    x
    (list-tail (cdr x) (- k 1))))

(define (list-ref x k) (nth k x))

#|

(prl 
(number? true)
(number? 5.5)
(number? 5)
(number? "5")
(number? '5)
(number? S)
(number? 'E)
(number? '(1 2 3 4))
)


(prl 
  (let ((x 2) (y 3))
      (let* ((x 7)
          (z (+ x y)))
          (* z x)))) ; 70

(prl 
  (let ((x 2) (y 3))
    (let ((x 7) (z (+ x y)))
          (* z x)))); 35

(prl           
(let ((name1 'x)
      (name2 'y))
      `(a `(b ,,name1 ,',name2 d) e)))
      
|#
 
#|  
(prl 
(symbol? 'foo) ;=) #t
(symbol? (car '(a b))) ;=) #t
(symbol? "bar") ;=) #f
(symbol? 'nil) ;=) #t
(symbol? '()) ;=) #f
(symbol? #f) ;=) #f
)
|#

(define (symbol->string symbol) (idtostring symboltable symbol))

;(prl 
;(symbol->string 'foo)
;(typeof (symbol->string 'foo))
;)

(define (string->symbol string) (stringtoid symboltable string))

;(prl 
;(string->symbol "hello")
;(typeof (string->symbol "hello"))
;)
